﻿// *****************************************************************
// *                                                               *
// *       Copyright (c) Fast Corporation, 2010                    *
// *                                                               *
// *   All Rights Reserved.  Unpublished rights  reserved  under   *
// *   the copyright laws of the Japan.                            *
// *                                                               *
// *   The software contained on this media  is  proprietary  to   *
// *   and  embodies  the  confidential  technology  of  Fast      *
// *   Corporation.  Possession, use,  duplication  or             *
// *   dissemination of the software and media is authorized only  *
// *   pursuant to a valid written license from Fast Corporation.  *
// *                                                               *
// *****************************************************************

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Reflection;
using System.Threading;

// -----------------------------------------------------------------
// FVIL Sample Program
// メインダイアログ
//
// 作成者 : N.Moriguchi
//
// 履歴 : 2011/02/24 初版
//
// FIO01 Sample Program
// -----------------------------------------------------------------
namespace FIO01_Sample
{
    public partial class FormFIO01_SAMPLE : Form
    {
        const int MAX_DIO_BIT = 32;
        const int HALF_DIO_BIT = 16;
        public FVIL.Dio.CFviDioFIO01 m_dio = new FVIL.Dio.CFviDioFIO01();
        public Thread m_thread;

        //コントロールの配列
        public Label[] m_di = new Label[HALF_DIO_BIT];
        public Label[] m_do = new Label[HALF_DIO_BIT];

        //デバイスのオープン状態
        private bool m_flgOpen = false;
        //エラー番号格納変数
        private int intErr;

        public FormFIO01_SAMPLE()
        {
            InitializeComponent();
        }

        /// <summary>
        /// メインフォーム初期化
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// メインフォームの初期化処理。
        /// </remarks>
        private void FormFIO01_SAMPLE_Load(object sender, EventArgs e)
        {
            // FVILの初期化
            // アプリケーションを起動した後、 この関数を呼び出してライブラリを初期化してください。 
            //  この時、ライセンスの認証が行われます。試用期間中であっても、この処理は必ず行う必要が有ります。 
            // 初期化を行わなかった場合は、プロテクトされたクラスの実行ができません。
            FVIL._SetUp.InitVisionLibrary();

            //コントロールの配列化
            for (int i = 0; i < HALF_DIO_BIT; i++)
            {
                string fieldname_di = string.Format("labelDI{0:d2}", i);
                this.m_di[i] = (Label)this.GetType().GetField(fieldname_di, BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.DeclaredOnly).GetValue(this);

                string fieldname_do = string.Format("labelDO{0:d2}", i);
                this.m_do[i] = (Label)this.GetType().GetField(fieldname_do, BindingFlags.Public | BindingFlags.NonPublic | BindingFlags.Instance | BindingFlags.DeclaredOnly).GetValue(this);
            }
            // DOボタンのCLICKイベント追加
            foreach (Label lb in m_do)
                lb.Click += new EventHandler(DO_Button_Click);

            this.Show();

            //デバイスIDを取得する
            FormOpen form_open = new FormOpen();
            if (form_open.ShowDialog(this) == DialogResult.OK)
            {
                try
                {
                    //デバイスオープン
                    m_dio.Open(form_open.m_id);
                    m_flgOpen = true;

                    //初期状態の取得
                    //入力ポートの読み出し
                    System.Collections.BitArray bitarray = m_dio.Read();
                    if (bitarray.Count == MAX_DIO_BIT)
                    {
                        for (int i = 0; i < HALF_DIO_BIT; i++)
                        {
                            if (bitarray[i] == true)
                            {
                                m_di[i].BackColor = Color.FromArgb(0xFF, 0x80, 0xFF);
                                m_di[i].Text = "ON";
                            }
                            else
                            {
                                m_di[i].BackColor = Color.Gray;                         //(0x80, 0x80, 0x80)
                                m_di[i].Text = "OFF";
                            }
                        }
                    }
                    //出力データの取得
                    bitarray = m_dio.GetWrite();
                    if (bitarray.Count == MAX_DIO_BIT)
                    {
                        for (int i = 0; i < HALF_DIO_BIT; i++)
                        {
                            if (bitarray[i] == true)
                            {
                                m_do[i].BackColor = Color.Aqua;                         //(0x00, 0xFF, 0xFF)
                                m_do[i].Text = "ON";
                            }
                            else
                            {
                                m_do[i].BackColor = Color.Gray;                         //(0x80, 0x80, 0x80)
                                m_do[i].Text = "OFF";
                            }
                        }
                    }

                    //割り込みエッジ極性の取得
                    if (m_dio.InterruptEdge == FVIL.Dio.InterruptEdge.Fall)
                    {
                        ComboBoxInterruptEdge.SelectedIndex = 0;
                    }
                    else
                    {
                        ComboBoxInterruptEdge.SelectedIndex = 1;
                    }

                    //Readタイマーのスタート
                    timer1.Start();

                    m_dio.Timeout = -1;
                }
                catch (FVIL.CFviException)
                {
                    //エラー処理
                    intErr = m_dio.ErrorCode;
                    MessageBox.Show(String.Format("ErrorCode : {0}", intErr, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error));
                    if (m_flgOpen == false)
                    {
                        //デバイスオープンできなかった場合はサンプルプログラムの動作を終了
                        this.Close();
                        return;
                    }
                }
            }
            else
            {
                //OKが押されなかった場合はサンプルプログラムの動作を終了
                this.Close();
                return;
            }
        }

        /// <summary>
        /// DOボタン
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// DOボタンのCLICKイベント。
        /// </remarks>
        private void DO_Button_Click(object sender, EventArgs e)
        {
            uint no = 0xFFFFFFFF;
            for (int i = 0; i < m_do.Length; i++)
            {
                if (m_do[i].Equals(sender))
                {
                    no = (uint)i;
                    break;
                }
            }
            if (no != 0xFFFFFFFF)
            {
                bool now_data = m_dio.GetWriteBit(no);
                m_dio.WriteBit(no, !now_data);
                if (no >= HALF_DIO_BIT)
                {
                    return;
                }
                if (!now_data)
                {
                    m_do[no].BackColor = Color.Aqua;                         //(0x00, 0xFF, 0xFF)
                    m_do[no].Text = "ON";
                }
                else
                {
                    m_do[no].BackColor = Color.Gray;                         //(0x80, 0x80, 0x80)
                    m_do[no].Text = "OFF";
                }
            }
        }

        /// <summary>
        /// 終了処理
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// アプリケーション終了処理。
        /// </remarks>
        private void FormFIO01_SAMPLE_FormClosing(object sender, FormClosingEventArgs e)
        {
            timer1.Enabled = false;
            if (m_flgOpen == true)
            {
                m_dio.Abort();
                System.Threading.Thread.Sleep(200);
            }
        }

        /// <summary>
        /// Readタイマー
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// タイマーで入力ポートの読み出し。
        /// </remarks>
        private void timer1_Tick(object sender, EventArgs e)
        {
            System.Collections.BitArray bitarray = m_dio.Read();
            if (bitarray.Count == MAX_DIO_BIT)
            {
                for (int i = 0; i < HALF_DIO_BIT; i++)
                {
                    if (bitarray[i] == true)
                    {
                        m_di[i].BackColor = Color.FromArgb(0xFF, 0x80, 0xFF);
                        m_di[i].Text = "ON";
                    }
                    else
                    {
                        m_di[i].BackColor = Color.Gray;                         //(0x80, 0x80, 0x80)
                        m_di[i].Text = "OFF";
                    }
                }
            }
        }

        /// <summary>
        /// スレッド
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 割り込みでInput値を表示する。
        /// </remarks>
        private void IntThreadRun()
        {
            System.Collections.BitArray bitarray;
            DeleSetIntText dele = SetIntText;
            UInt32 data;
            UInt32 one = 1;

            while (true)
            {
                try
                {
                    data = 0;
                    bitarray = m_dio.ReadInterrupt();
                    for (int i = 0; i < HALF_DIO_BIT; i++)
                    {
                        if (m_dio.ReadBit((uint)i) == true)
                        {
                            data |= one << i;
                        }
                    }
                    this.Invoke(dele, data, Color.FromArgb(0xFF, 0x80, 0xFF));
                    System.Threading.Thread.Sleep(200);
                    this.Invoke(dele, data, Color.Silver);                          //(0xC0, 0xC0, 0xC0)
                }
                catch (FVIL.CFviException)
                {
                    //エラー処理
                    intErr = m_dio.ErrorCode;
                    if (intErr != FVIL.Dio.ErrorCode.ABORT)
                    {
                        MessageBox.Show(String.Format("ErrorCode : {0}", intErr, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error));
                    }
                    break;
                }
            }
        }

        /// <summary>
        /// 割り込み監視チェックボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// スレッドによる割り込み監視処理。
        /// </remarks>
        private void CheckBoxCheckInterrupt_CheckedChanged(object sender, EventArgs e)
        {
            if (CheckBoxCheckInterrupt.Checked == true)
            {
                ComboBoxInterruptEdge.Enabled = false;
                //スレッド生成
                m_thread = new Thread(new ThreadStart(IntThreadRun));
                //Threadのスタート
                m_thread.Start();
            }
            else
            {
                m_dio.Abort();
                ComboBoxInterruptEdge.Enabled = true;
            }
        }

        /// <summary>
        /// デリゲート
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// スレッドでテキストボックスを変更する。
        /// </remarks>
        delegate void DeleSetIntText(uint int_di, Color col);
        void SetIntText(uint int_di, Color color)
        {
            TextBoxInterrupt.Text = string.Format("0x{0:x8}", int_di);
            TextBoxInterrupt.BackColor = color;
        }

        /// <summary>
        /// 割り込みエッジ極性コンボボックス
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        /// <remarks>
        /// 割り込み信号のエッジ極性の設定処理。
        /// </remarks>
        private void ComboBoxInterruptEdge_SelectionChangeCommitted(object sender, EventArgs e)
        {
            try
            {
                if (ComboBoxInterruptEdge.SelectedIndex == 0)
                {
                    m_dio.InterruptEdge = FVIL.Dio.InterruptEdge.Fall;
                }
                else
                {
                    m_dio.InterruptEdge = FVIL.Dio.InterruptEdge.Rise;
                }
            }
            catch (FVIL.CFviException)
            {
                //エラー処理
                intErr = m_dio.ErrorCode;
                MessageBox.Show(String.Format("ErrorCode : {0}", intErr, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error));
            }
        }

    }
}